'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.nodeFieldConfig = exports.TypeMapForRelayNode = undefined;
exports.composeWithRelay = composeWithRelay;
exports._composeWithRelay = _composeWithRelay;

var _graphqlCompose = require('graphql-compose');

var _graphql = require('graphql-compose/lib/graphql');

var _nodeInterface = require('./nodeInterface');

var _nodeInterface2 = _interopRequireDefault(_nodeInterface);

var _wrapMutationResolver = require('./wrapMutationResolver');

var _wrapMutationResolver2 = _interopRequireDefault(_wrapMutationResolver);

var _globalId = require('./globalId');

var _nodeFieldConfig = require('./nodeFieldConfig');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// all wrapped typeComposers with Relay, stored in this variable
// for futher type resolving via NodeInterface.resolveType method

/* eslint-disable no-use-before-define */

var TypeMapForRelayNode = exports.TypeMapForRelayNode = {};
var nodeFieldConfig = exports.nodeFieldConfig = (0, _nodeFieldConfig.getNodeFieldConfig)(TypeMapForRelayNode);

function composeWithRelay(typeComposer) {
  return _composeWithRelay(typeComposer);
}

function _composeWithRelay(tc) {
  if (!tc || tc.constructor.name !== 'TypeComposer') {
    throw new Error('You should provide TypeComposer instance to composeWithRelay method');
  }

  if (tc.getTypeName() === 'Query' || tc.getTypeName() === 'RootQuery') {
    tc.setField('node', nodeFieldConfig);
    return tc;
  }

  if (tc.getTypeName() === 'Mutation' || tc.getTypeName() === 'RootMutation') {
    // just skip
    return tc;
  }

  if (!tc.hasRecordIdFn()) {
    throw new Error(`TypeComposer(${tc.getTypeName()}) should have recordIdFn. ` + 'This function returns ID from provided object.');
  }

  var findById = tc.getResolver('findById');
  if (!findById) {
    throw new Error(`TypeComposer(${tc.getTypeName()}) provided to composeWithRelay ` + 'should have findById resolver.');
  }
  TypeMapForRelayNode[tc.getTypeName()] = {
    resolver: findById,
    tc
  };

  tc.addFields({
    id: {
      type: new _graphql.GraphQLNonNull(_graphql.GraphQLID),
      description: 'The globally unique ID among all types',
      resolve: function resolve(source) {
        return (0, _globalId.toGlobalId)(tc.getTypeName(), tc.getRecordId(source));
      }
    }
  });

  tc.addInterface(_nodeInterface2.default);

  tc.getResolvers().forEach(function (resolver, resolverName) {
    if (resolver.kind === 'mutation') {
      var wrappedResolver = (0, _wrapMutationResolver2.default)(resolver, {
        resolverName,
        rootTypeName: tc.getTypeName()
      });
      tc.setResolver(resolverName, wrappedResolver);
    }
  });

  return tc;
}